/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.common;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import kieker.analysis.display.annotation.Display;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.plugin.annotation.Property;
import kieker.analysis.plugin.annotation.RepositoryPort;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.analysis.plugin.reader.AbstractReaderPlugin;
import kieker.analysis.repository.AbstractRepository;
import kieker.analysis.repository.annotation.Repository;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.exception.ProjectLoadException;

/**
 * The {@link ClassAndMethodContainer} is a container which contains - as the name already suggests - various classes and methods. To be more precisely, it uses a
 * given class loader to load the equivalence of specific classes within this application to ensure that comparisons, assignments and the use of specific methods
 * will be done correctly. This is necessary as for every project within this application there will be a number of libraries which will be combined in one class
 * loader. This will result in multiple version of one and the same class and therefore in problems, if one doesn't use the correct class version.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
public final class ClassAndMethodContainer {

	/**
	 * This is the logger, which will be used to log messages and exceptions for this class.
	 */
	private static final Log LOG = LogFactory.getLog(ClassAndMethodContainer.class);
	/**
	 * The exception which is used if something went wrong during loading.
	 */
	private static final String MSG_LOAD_EXCEPTION = "An error occured while loading the classes and methods.";

	/**
	 * This is the class equivalence of {@link AbstractRepository}.
	 */
	private final Class<?> abstractRepositoryClass;
	/**
	 * This is the class equivalence of {@link AbstractPlugin}.
	 */
	private final Class<?> abstractPluginClass;
	/**
	 * This is the class equivalence of {@link AbstractFilterPlugin}.
	 */
	private final Class<?> abstractFilterPluginClass;
	/**
	 * This is the class equivalence of {@link AbstractRepository}.
	 */
	private final Class<?> abstractReaderPluginClass;
	/**
	 * This is the class equivalence of {@link Plugin}.
	 */
	private final Class<? extends Annotation> pluginAnnotationClass;
	/**
	 * This is the class equivalence of {@link Repository}.
	 */
	private final Class<? extends Annotation> repositoryAnnotationClass;
	/**
	 * This is the class equivalence of {@link Property}.
	 */
	private final Class<? extends Annotation> propertyAnnotationClass;
	/**
	 * This is the class equivalence of {@link OutputPort}.
	 */
	private final Class<? extends Annotation> outputPortAnnotationClass;
	/**
	 * This is the class equivalence of {@link InputPort}.
	 */
	private final Class<? extends Annotation> inputPortAnnotationClass;
	/**
	 * This is the class equivalence of {@link RepositoryPort}.
	 */
	private final Class<? extends Annotation> repositoryPortAnnotationClass;
	/**
	 * This is the class equivalence of {@link Display}.
	 */
	private final Class<? extends Annotation> displayAnnotationClass;
	/**
	 * This is the description()-method of the class equivalence of {@link Plugin}.
	 */
	private final Method pluginDescriptionMethod;
	/**
	 * This is the description()-method of the class equivalence of {@link Repository}.
	 */
	private final Method repositoryDescriptionMethod;
	/**
	 * This is the configuration()-method of the class equivalence of {@link Plugin}.
	 */
	private final Method pluginConfigurationMethod;
	/**
	 * This is the description()-method of the class equivalence of {@link Repository}.
	 */
	private final Method repositoryConfigurationMethod;
	/**
	 * This is the outputPorts()-method of the class equivalence of {@link Plugin}.
	 */
	private final Method pluginOutputPortsMethod;
	/**
	 * This is the repositoryPorts()-method of the class equivalence of {@link Plugin}.
	 */
	private final Method pluginRepositoryPortsMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link Display}.
	 */
	private final Method displayNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link InputPort}.
	 */
	private final Method inputPortNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link OutputPort}.
	 */
	private final Method outputPortNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link RepositoryPort}.
	 */
	private final Method repositoryPortNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link Property}.
	 */
	private final Method propertyNameMethod;
	/**
	 * This is the defaultValue()-method of the class equivalence of {@link Property}.
	 */
	private final Method propertyDefaultValueMethod;

	/**
	 * Creates a new instance of this class, using the given class loader.
	 * 
	 * @param classLoader
	 *            The class loader which will be used to load all classes and determine their methods.
	 * @throws ProjectLoadException
	 *             If one or more of the classes or methods for this container could not be found.
	 */
	@SuppressWarnings("unchecked")
	public ClassAndMethodContainer(final ClassLoader classLoader) throws ProjectLoadException {
		try {
			this.abstractFilterPluginClass = classLoader.loadClass(AbstractFilterPlugin.class.getCanonicalName());
			this.abstractReaderPluginClass = classLoader.loadClass(AbstractReaderPlugin.class.getCanonicalName());
			this.abstractRepositoryClass = classLoader.loadClass(AbstractRepository.class.getCanonicalName());
			this.abstractPluginClass = classLoader.loadClass(AbstractPlugin.class.getCanonicalName());

			this.pluginAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Plugin.class.getCanonicalName());
			this.repositoryAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Repository.class.getCanonicalName());
			this.propertyAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Property.class.getCanonicalName());
			this.outputPortAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(OutputPort.class.getCanonicalName());
			this.inputPortAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(InputPort.class.getCanonicalName());
			this.repositoryPortAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(RepositoryPort.class.getCanonicalName());
			this.displayAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Display.class.getCanonicalName());

			this.pluginDescriptionMethod = this.pluginAnnotationClass.getMethod("description", new Class<?>[0]);
			this.repositoryDescriptionMethod = this.repositoryAnnotationClass.getMethod("description", new Class<?>[0]);
			this.pluginConfigurationMethod = this.pluginAnnotationClass.getMethod("configuration", new Class<?>[0]);
			this.repositoryConfigurationMethod = this.repositoryAnnotationClass.getMethod("configuration", new Class<?>[0]);
			this.pluginOutputPortsMethod = this.pluginAnnotationClass.getMethod("outputPorts", new Class<?>[0]);
			this.pluginRepositoryPortsMethod = this.pluginAnnotationClass.getMethod("repositoryPorts", new Class<?>[0]);
			this.displayNameMethod = this.displayAnnotationClass.getMethod("name", new Class<?>[0]);
			this.inputPortNameMethod = this.inputPortAnnotationClass.getMethod("name", new Class<?>[0]);
			this.outputPortNameMethod = this.outputPortAnnotationClass.getMethod("name", new Class<?>[0]);
			this.repositoryPortNameMethod = this.repositoryPortAnnotationClass.getMethod("name", new Class<?>[0]);
			this.propertyNameMethod = this.propertyAnnotationClass.getMethod("name", new Class<?>[0]);
			this.propertyDefaultValueMethod = this.propertyAnnotationClass.getMethod("defaultValue", new Class<?>[0]);
		} catch (final ClassNotFoundException ex) {
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		} catch (final NoSuchMethodException ex) {
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		} catch (final SecurityException ex) {
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		} catch (final ClassCastException ex) {
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		}
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractFilterPluginClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractFilterPluginClass() {
		return this.abstractFilterPluginClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractReaderPluginClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractReaderPluginClass() {
		return this.abstractReaderPluginClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getPluginAnnotationClass() {
		return this.pluginAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getRepositoryAnnotationClass() {
		return this.repositoryAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getPropertyAnnotationClass() {
		return this.propertyAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#outputPortAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getOutputPortAnnotationClass() {
		return this.outputPortAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#inputPortAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getInputPortAnnotationClass() {
		return this.inputPortAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryPortAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getRepositoryPortAnnotationClass() {
		return this.repositoryPortAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#displayAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getDisplayAnnotationClass() {
		return this.displayAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginDescriptionMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginDescriptionMethod() {
		return this.pluginDescriptionMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryDescriptionMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getRepositoryDescriptionMethod() {
		return this.repositoryDescriptionMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginConfigurationMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginConfigurationMethod() {
		return this.pluginConfigurationMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryConfigurationMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getRepositoryConfigurationMethod() {
		return this.repositoryConfigurationMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginOutputPortsMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginOutputPortsMethod() {
		return this.pluginOutputPortsMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginRepositoryPortsMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginRepositoryPortsMethod() {
		return this.pluginRepositoryPortsMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#displayNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getDisplayNameMethod() {
		return this.displayNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractRepositoryClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractRepositoryClass() {
		return this.abstractRepositoryClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractPluginClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractPluginClass() {
		return this.abstractPluginClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#inputPortNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getInputPortNameMethod() {
		return this.inputPortNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#outputPortNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getOutputPortNameMethod() {
		return this.outputPortNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryPortNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getRepositoryPortNameMethod() {
		return this.repositoryPortNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPropertyNameMethod() {
		return this.propertyNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyDefaultValueMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPropertyDefaultValueMethod() {
		return this.propertyDefaultValueMethod;
	}

}
